/*************************************************************************************************/
/*! \file
\ingroup	Example Sequence
\brief		This sequence is intended to illustrate how to calculate a position from analog input
			and go to this position.
			
			This particular sequence is for the K17a, but the idea stays the same for other
			controller boards. Not all have analog input and/or analog output.

CREATED
\date		25.04.2023
\version	1.0.0
\author		T.Durot/Adlos AG
**************************************************************************************************/


#include "L0_TA_datatypes_GCC_ARM.h"			// MUST NOT BE CHANGED!!!
#include "stm32f0xx.h"							// MUST NOT BE CHANGED!!!
#include "L0_KannMotDrvTypes.h"					// MUST NOT BE CHANGED!!!
#include "L2_APPC_SPS_User.h"					// MUST NOT BE CHANGED!!!
#include "L2_APPC_SPS_myUserTypes.h"			// MUST NOT BE CHANGED!!!
#include "L2_APPC_SPS_myUserFunctionDefines.h"	// MUST NOT BE CHANGED!!!


// ---- SPS Control-Block ----------------------
volatile const LOCATEUSCNTRL  tAPPCSPS_CNTRLBLK   CNTRLBLKINIT = {
                                                .stCntrl.UserStandardFuncPtr_1=AppCSPS_USER_SEQ_STANDARD_1,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.UserStandardFuncPtr_2=AppCSPS_USER_SEQ_STANDARD_2,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.UserErrorFuncPtr=AppCSPS_USER_SEQ_ERROR,				// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.u32_BlkChkSum=0xFFFFFFFF,								// MUST NOT BE CHANGED!!! -> Ckecksum
                                                #ifdef fFeEXTRAFUNC
                                                #ifdef fFeCANRXTX
                                                .stCntrl.UserRxEventFuncPtr=AppCSPS_USER_SEQ_CAN_RX_Event,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.UserTxEventFuncPtr=AppCSPS_USER_SEQ_CAN_TX_SynxEvent,	// MUST NOT BE CHANGED!!! -> Func Pointer
                                                #endif
                                                #ifdef fFeRS232RxEvent
                                                .stCntrl.UserRxEventFuncPtr=AppCSPS_USER_SEQ_232_RX_Event,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                #endif
                                                #ifdef fFe100msEvent
                                                .stCntrl.User100msEventFuncPtr=AppCSPS_USER_100ms_Event,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                #endif
                                                #endif
                                                .stInfo.su.u16_UserPrgVersion=0x1000,							// VERSION of sequence on device
                                                .stInfo.su.u16_UserReserved=0x0000,								// MUST NOT BE CHANGED!!!
                                                .stInfo.su.u8_UserPrgTxt={"Example: GoToPos Ain K17a"}		// DESCRIPTION of sequence on device
                                                };


// ---- Globale Variabeln loesen -----------------
LOCATEVARSPS tAPPCSPS stAppCSPS;

//###################################################################
//#		User defines, variables and more for better readability		#
//###################################################################
// DETAILS for linearizing analog input to position
// --> P1(0/0) P2(1129/-3600)	1129: 10000mV / 8.86mVpE
// --> y=-3.189x=(-319x)/100
#define MIN_POS						0
#define MAX_POS						-3600
//###################################################################
#define GetPosFromAin(Ain)			(-319*Ain)/100		// see DETAILS
#define POS_TOLERANCE				50
#define i32_ActualAin				UVar.i32_PosVAR_um_01deg[0]
#define i32_CalcVar					UVar.i32_PosVAR_um_01deg[1]
#define i32_NextTarget				UVar.KxxFlexUser.i32_UserVar[0]
#define u08_state					UVar.u8_StepChain[0]

typedef enum
{
	eSTATE__GETAIN		= 0,
	eSTATE__CALCNEXTPOS	= 1,
	eSTATE__GOTOPOS		= 2
}
tenState;


/************************************************************************************************/
/*!
 * \brief	SPS-USER function / TaskHandler1 (first)
 * \details	Is called while <SPS-RUN> state every 5ms (not while in error-State!).
 *			Your code shall not block (Cooperative Multitask).
 *			Execution time of your block shall be < 50us / max 0.5ms!
 *
 *			Program here, regularly things, e.g. Checking IO's ....
**************************************************************************************************/
void AppCSPS_USER_SEQ_STANDARD_1(void)
{
	switch(u08_state)
	{
		case eSTATE__GETAIN:
		{
			// Get analog input
			i32_ActualAin = UVar.KxxFlexUser.K17a.u16_Ain0to24_8d86mVpE;

			u08_state = eSTATE__CALCNEXTPOS;
			break;
		}
		case eSTATE__CALCNEXTPOS:
		{
			// Calculate next position
			i32_NextTarget = GetPosFromAin(i32_ActualAin);
			
			u08_state = eSTATE__GOTOPOS;
			break;
		}
		case eSTATE__GOTOPOS:
		{
			// Check position tolerance
			i32_CalcVar = i32_NextTarget - UVar.i32_ActualPos_um_01deg;
			// Difference to last position as absolute value
			if(i32_CalcVar < 0)
			{
				i32_CalcVar *= -1;
			}
			// If differenz outside tolerance, call go to pos command
			if(i32_CalcVar > (POS_TOLERANCE))
			{
				if(UFu_GotoFuncP(i32_NextTarget, eGOTO_um_01deg) == eMS_OK)
				{
					// do something
				}
			}
			
			u08_state = eSTATE__GETAIN;
			break;
		}
		default:
		{
			u08_state = eSTATE__GETAIN;
			break;
		}
	}
}

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / TaskHandler2 (second)
 * \details	Is called while <SPS-RUN> state every 5ms (not while in error-State!).
 *			Your code shall not block (Cooperative Multitask).
 *			Execution time of your block shall be < 50us / max 0.5ms!
 *
 *			TaskHandler2, is called 1ms later than TaskHandler1, to allow CPU load splitting into 2 parts.
 *
 *			Program here, regularly things, e.g. Checking IO's ....
**************************************************************************************************/
void AppCSPS_USER_SEQ_STANDARD_2(void)
{
	// Fill in code...
	mNOP();
}

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / ErrorHandler
 * \details	Is called once, when core enters <ERROR> state.
 *			Depending on Error, it is not allowed to start here a Moving CMD!
 *
 *			Program here, things to show error, e.g. switch_ON Error-Output
**************************************************************************************************/
void AppCSPS_USER_SEQ_ERROR(void)
{
	// Fill in code...
	mNOP();
}

#ifdef fFeRS232RxEvent
/************************************************************************************************/
/*!
 * \brief	SPS-USER function / RS232-RxEvent on receive of [Df] Command
 * \details	If you react on RS232 Data here, you shall also Clear data receive Structure at the end of the Handler.
 *			To confirm the main firmware Command was handled by your code.
 *
 *			You might use stAppCSPS.SPSUserVar.st32_ComData.u32_data instead of pComData->
 *			pComData->   points to stAppCSPS.SPSUserVar.st32_ComData.u32_data
 
 *			Serial cmd e.g.:	Df#0D#XL4[0x12345678]#CK 				(COMWatch)
 *								0x44 0x66 0x0D 0x78 0x56 0x34 0x12 0x35	(HEX)
 *
 *			Program here your own Command interpreter...
 *
 * \param	pComData - pointer to data received from serial port received data on command [Df]
**************************************************************************************************/
void AppCSPS_USER_SEQ_232_RX_Event(tCSPSCOMRXDATA* pComData)	//!< RW: from serial port received data command <Df>, shall be cleared by USER !)
{
   // Fill in code...
   mNOP();

   // Set
   // pComData->u32_data = 0;           // Clear Data as Handled information
}
#endif

#ifdef fFeCANRXTX
/************************************************************************************************/
/*!
 * \brief	SPS-USER function / CAN-RXEvent on CANopen-PDO4 Rx
 * \details	e.g. MsgId=  0x508  DLC=8  Data = ???
 *
 *			Program here your own command interpreter...
 * 
 * \param	pRxData - Pointer to Data received
 * \param	Datalength - Data count [0..8]
**************************************************************************************************/
void AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
{
   // Fill in code...
   mNOP();
   /*
   // .. My Command interpreter Example
   switch (pRxData->u08_Data[0])
   {
     // CMD=0: Goto-Pos, Target Position is in
     case 00:
     {
        if (Datalength!=8)
        {
           return;
        }
        if (stAppCSPS.SPSCallFunctions.GotoFuncP((SI_32) pRxData->u32_Data[1], eGOTO_um_01deg)==eMS_OK) // Call Goto Function
        {
          stAppCSPS.SPSUserVar.u16_Timer5ms[0]=800;                // 4s -> 5ms x 400
          stAppCSPS.SPSUserVar.u8_StepChain[0]=4;                  // next Step = Delay
          stAppCSPS.SPSCallFunctions.COM_MoveAllow(false);         // CAN CMDs die eine Bewegung auslsen blocken
        }
       break;
     }
     // CMD=2: Query... we send here a PDO4 asynchronous back
     case 02:
     {
       tCANDATA CanTxData;
       CanTxData.u32_Data[0]=0x01234567;
       CanTxData.u32_Data[1]=0x89ABCDEF;
       if (stAppCSPS.SPSCallFunctions.CANopen_PDO4_Send(&CanTxData,8))
       {
       }
       break;
     }
   }
   */
}
#endif

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / CAN-TxSyncEvent on CANopen-PDO4 Tx (Sync on Time OR Sync CMD reception)
 * \details	Collect here your data to send officially at your PDO4-Transaction,
 *			especially if it shall be transmitted synchronized!
 *
 *			Program here your own PDO4-Tx (synchronized Message collector)...
**************************************************************************************************/
#ifdef fFeCANRXTX
void AppCSPS_USER_SEQ_CAN_TX_SynxEvent(void)
{
   // Example Preparation of PDO4-Tx ( Answer )
   /*tCANDATA CanTxData;
   CanTxData.u32_Data[0]=0x01234567;
   CanTxData.u32_Data[1]=0x89ABCDEF;
   stAppCSPS.SPSCallFunctions.CANopen_PDO4_Send(&CanTxData,8);*/
   // Fill in code...
   mNOP();
}
#endif

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / 100ms Regular Event, called even while in error or homing state
 * \details	Do not define moving actions here... use it e.g. for LED or output signaling
 *
 *			Program here your own LED/output/Transmit Management...
**************************************************************************************************/
#ifdef fFe100msEvent
void AppCSPS_USER_100ms_Event(void)
{
	// Fill in code...
   mNOP();
}
#endif

