/*************************************************************************************************/
/*! \file
\ingroup	Example Sequence
\brief		This sequence is intended to illustrate how to init config,
			homing (stall, input) and set actual position to 0.
			
			Init with setting of configuration.
			Then homing on stall or Din and set this as 0 position.
			Then change between 2 positions every 2s.

CREATED
\date		24.04.2023
\version	1.0.0
\author		T.Durot/Adlos AG
**************************************************************************************************/


#include "L0_TA_datatypes_GCC_ARM.h"			// MUST NOT BE CHANGED!!!
#include "stm32f0xx.h"							// MUST NOT BE CHANGED!!!
#include "L0_KannMotDrvTypes.h"					// MUST NOT BE CHANGED!!!
#include "L2_APPC_SPS_User.h"					// MUST NOT BE CHANGED!!!
#include "L2_APPC_SPS_myUserTypes.h"			// MUST NOT BE CHANGED!!!
#include "L2_APPC_SPS_myUserFunctionDefines.h"	// MUST NOT BE CHANGED!!!


// ---- SPS Control-Block ----------------------
volatile const LOCATEUSCNTRL  tAPPCSPS_CNTRLBLK   CNTRLBLKINIT = {
                                                .stCntrl.UserStandardFuncPtr_1=AppCSPS_USER_SEQ_STANDARD_1,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.UserStandardFuncPtr_2=AppCSPS_USER_SEQ_STANDARD_2,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.UserErrorFuncPtr=AppCSPS_USER_SEQ_ERROR,				// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.u32_BlkChkSum=0xFFFFFFFF,								// MUST NOT BE CHANGED!!! -> Ckecksum
                                                #ifdef fFeEXTRAFUNC
                                                #ifdef fFeCANRXTX
                                                .stCntrl.UserRxEventFuncPtr=AppCSPS_USER_SEQ_CAN_RX_Event,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.UserTxEventFuncPtr=AppCSPS_USER_SEQ_CAN_TX_SynxEvent,	// MUST NOT BE CHANGED!!! -> Func Pointer
                                                #endif
                                                #ifdef fFeRS232RxEvent
                                                .stCntrl.UserRxEventFuncPtr=AppCSPS_USER_SEQ_232_RX_Event,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                #endif
                                                #ifdef fFe100msEvent
                                                .stCntrl.User100msEventFuncPtr=AppCSPS_USER_100ms_Event,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                #endif
                                                #endif
                                                .stInfo.su.u16_UserPrgVersion=0x1000,							// VERSION of sequence on device
                                                .stInfo.su.u16_UserReserved=0x0000,								// MUST NOT BE CHANGED!!!
                                                .stInfo.su.u8_UserPrgTxt={"Example: Init, Homing, GoToPos"}		// DESCRIPTION of sequence on device
                                                };


// ---- Globale Variabeln loesen -----------------
LOCATEVARSPS tAPPCSPS stAppCSPS;

//###################################################################
//#		User defines, variables and more for better readability		#
//###################################################################
#define POSITION_1			500		// Position 1
#define POSITION_2			6000	// Position 2
#define POSITION_TOL		20		// Position tolerance
#define DELAY_GOTOPOS		400		// Delay between go to pos. 400*5ms=2000ms

#define HOMING_ON_STALL		1		// 1: on stall; 0: on input

#define u08_state			UVar.u8_StepChain[0]		// State
#define u16_default_timer	UVar.u16_Timer5ms[0]		// Default timer
#define u16_delay_timer		UVar.u16_Timer5ms[1]		// Delay timer
#define i32_next_pos		UVar.i32_PosVAR_um_01deg[0]	// Next position
#define i32_calc_var		UVar.i32_PosVAR_um_01deg[1]	// Calculation variable
#define i32_act_pos			UVar.i32_ActualPos_um_01deg	// Actual position

// ---- States --------------------
typedef enum
{
   eSTATE__INIT_CONFIG	= 0,	// Init: Device configuration
   eSTATE__INIT_HOMING	= 1,	// Init: Do homing on stall
   eSTATE__INIT_SET0POS	= 2,	// Init: Set 0 position
   eSTATE__GOTOPOS		= 10,	// Seq: Go to pos with tolerance
   eSTATE__DELAY		= 99,	// Wait delay
};


/************************************************************************************************/
/*!
 * \brief	SPS-USER function / TaskHandler1 (first)
 * \details	Is called while <SPS-RUN> state every 5ms (not while in error-State!).
 *			Your code shall not block (Cooperative Multitask).
 *			Execution time of your block shall be < 50us / max 0.5ms!
 *
 *			Program here, regularly things, e.g. Checking IO's ....
**************************************************************************************************/
void AppCSPS_USER_SEQ_STANDARD_1(void)
{
	// Initialisierung
	switch(u08_state)
	{
		case eSTATE__INIT_CONFIG:
		{
			// Config parameters
			UFu_DrvChgSetting(12, 600);			// Min. Speed	[0.1rpm]
			UFu_DrvChgSetting(13, 3000);		// Max. Speed	[0.1rpm]
			UFu_DrvChgSetting(14, 20);			// Torque-Hold	[0.5.%]					
			UFu_DrvChgSetting(15, 200);			// Torque-ACC	[0.5.%]
			UFu_DrvChgSetting(16, 200);			// Torque-RUN	[0.5.%]
			UFu_DrvChgSetting(17, 200);			// Torque-DEC	[0.5.%]
			UFu_DrvChgSetting(18, 160);			// Acceleration	[0.5%]
			UFu_DrvChgSetting(19, 200);			// Deceleration	[0.5.%]
			UFu_DrvChgSetting(21, 0b00000000);	// Filter Din/Ain
			
			i32_next_pos = POSITION_1;	// Assign first position

			u08_state++;
			break;
		}
		case eSTATE__INIT_HOMING:
		{
#if (HOMING_ON_STALL)
			// Homing until stall, torque 100%
			stAppCSPS.pHomingCntrl->enMode = Homing_tilSTALL | 0x0F;	// Change to homing mode, with torque 0x0F (100%)
#endif
#if (!HOMING_ON_STALL)
			// Homing until input 4 hi single move, torque 100%
			stAppCSPS.pHomingCntrl->enMode = Homing_DIx_gLo_sm | 0x04;	// Change to homing mode, with DIn 4
#endif
			stAppCSPS.pHomingCntrl->u16_Timeout_ms = 10000;
			if (UFu_HomingPrepFuncP(-20) == eMS_OK)	// Call homing function with -20rpm
			{
				u08_state++;
			}
			break;
		}
		case eSTATE__INIT_SET0POS:
		{
			// Check if motor HOLD or IDLE
			if (UVar.enDrvState <= eMS_HOLD)
			{
				// Set 0 position
				stAppCSPS.pHomingCntrl->enMode = Homing_StoreMarkasHome;
				if (UFu_HomingPrepFuncP(0)==eMS_OK)     // Call Homing Function
				{
					UVar.u16_Timer5ms[0] = 400;	// Set delay of 400*5ms=2s
					u08_state++;
				}
			}
		    break;
		}
		case eSTATE__GOTOPOS:
		{
			// Calc diff from wanted position to actual position
			i32_calc_var = i32_next_pos - i32_act_pos;

			// Diff as absolute value
			if(i32_calc_var < 0)
			{
				i32_calc_var *= -1;
			}
			// If position outside tolerance, call go to function
			if(i32_calc_var > POSITION_TOL)
			{
				if(UFu_GotoFuncP(i32_next_pos, eGOTO_um_01deg) == eMS_OK)
				{
				}
			}
			else
			{
				u16_delay_timer = DELAY_GOTOPOS;
				u08_state = eSTATE__DELAY;
			}
			break;
		}
		case eSTATE__DELAY:
		{
			// Wait for delay, then go to next pos case
			if (u16_delay_timer == 0)            
			{
				if(i32_next_pos == POSITION_1)
				{
					i32_next_pos = POSITION_2;
				}
				else if(i32_next_pos == POSITION_2)
				{
					i32_next_pos = POSITION_1;
				}
				else
				{
					// We should never get here
				}
				u08_state = eSTATE__GOTOPOS;
			}
			break;
		}
		default:
		{
			// We should only get here after initialisation is done, after set 0 pos
			if ((u16_default_timer == 0) && (u08_state == eSTATE__INIT_SET0POS+1))            
			{
				u08_state = eSTATE__GOTOPOS;
			}
			break;
		}
	}
}

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / TaskHandler2 (second)
 * \details	Is called while <SPS-RUN> state every 5ms (not while in error-State!).
 *			Your code shall not block (Cooperative Multitask).
 *			Execution time of your block shall be < 50us / max 0.5ms!
 *
 *			TaskHandler2, is called 1ms later than TaskHandler1, to allow CPU load splitting into 2 parts.
 *
 *			Program here, regularly things, e.g. Checking IO's ....
**************************************************************************************************/
void AppCSPS_USER_SEQ_STANDARD_2(void)
{
	// Fill in code...
	mNOP();
}

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / ErrorHandler
 * \details	Is called once, when core enters <ERROR> state.
 *			Depending on Error, it is not allowed to start here a Moving CMD!
 *
 *			Program here, things to show error, e.g. switch_ON Error-Output
**************************************************************************************************/
void AppCSPS_USER_SEQ_ERROR(void)
{
	// Fill in code...
	mNOP();
}

#ifdef fFeRS232RxEvent
/************************************************************************************************/
/*!
 * \brief	SPS-USER function / RS232-RxEvent on receive of [Df] Command
 * \details	If you react on RS232 Data here, you shall also Clear data receive Structure at the end of the Handler.
 *			To confirm the main firmware Command was handled by your code.
 *
 *			You might use stAppCSPS.SPSUserVar.st32_ComData.u32_data instead of pComData->
 *			pComData->   points to stAppCSPS.SPSUserVar.st32_ComData.u32_data
 
 *			Serial cmd e.g.:	Df#0D#XL4[0x12345678]#CK 				(COMWatch)
 *								0x44 0x66 0x0D 0x78 0x56 0x34 0x12 0x35	(HEX)
 *
 *			Program here your own Command interpreter...
 *
 * \param	pComData - pointer to data received from serial port received data on command [Df]
**************************************************************************************************/
void AppCSPS_USER_SEQ_232_RX_Event(tCSPSCOMRXDATA* pComData)	//!< RW: from Serial Port received data Command <Df>, shall be cleared by USER !)
{
   // Fill in code...
   mNOP();
}
#endif

#ifdef fFeCANRXTX
/************************************************************************************************/
/*!
 * \brief	SPS-USER function / CAN-RXEvent on CANopen-PDO4 Rx
 * \details	e.g. MsgId=  0x508  DLC=8  Data = ???
 *
 *			Program here your own command interpreter...
 * 
 * \param	pRxData - Pointer to Data received
 * \param	Datalength - Data count [0..8]
**************************************************************************************************/
void AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
{
   // Fill in code...
   mNOP();
}
#endif

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / CAN-TxSyncEvent on CANopen-PDO4 Tx (Sync on Time OR Sync CMD reception)
 * \details	Collect here your data to send officially at your PDO4-Transaction,
 *			especially if it shall be transmitted synchronized!
 *
 *			Program here your own PDO4-Tx (synchronized Message collector)...
**************************************************************************************************/
#ifdef fFeCANRXTX
void AppCSPS_USER_SEQ_CAN_TX_SynxEvent(void)
{
   // Fill in code...
   mNOP();
}
#endif

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / 100ms Regular Event, called even while in error or homing state
 * \details	Do not define moving actions here... use it e.g. for LED or output signaling
 *
 *			Program here your own LED/output/Transmit Management...
**************************************************************************************************/
#ifdef fFe100msEvent
void AppCSPS_USER_100ms_Event(void)
{
	// Fill in code...
   mNOP();
}
#endif

