/*************************************************************************************************/
/*! \file
\ingroup	Example Sequence
\brief		This sequence is intended to illustrate how a interact with serial commands.

			Rotate with speed received from serial command 'Df#0D#XL4[SPEED_CMD]#CK' (TaskHandler1)
			and send current position ('Df#0D#XL4[current pos]#CK') over serial interface every
			100ms if rotating (Taskhandler2).
			
			Stop:		SPEED_CMD 0x00
			Speed 1:	SPEED_CMD 0x01
			Speed 2:	SPEED_CMD 0x02
			Speed 3:	SPEED_CMD 0x03

CREATED
\date		25.04.2023
\version	1.0.0
\author		T.Durot/Adlos AG
**************************************************************************************************/


#include "L0_TA_datatypes_GCC_ARM.h"			// MUST NOT BE CHANGED!!!
#include "stm32f0xx.h"							// MUST NOT BE CHANGED!!!
#include "L0_KannMotDrvTypes.h"					// MUST NOT BE CHANGED!!!
#include "L2_APPC_SPS_User.h"					// MUST NOT BE CHANGED!!!
#include "L2_APPC_SPS_myUserTypes.h"			// MUST NOT BE CHANGED!!!
#include "L2_APPC_SPS_myUserFunctionDefines.h"	// MUST NOT BE CHANGED!!!


// ---- SPS Control-Block ----------------------
volatile const LOCATEUSCNTRL  tAPPCSPS_CNTRLBLK   CNTRLBLKINIT = {
                                                .stCntrl.UserStandardFuncPtr_1=AppCSPS_USER_SEQ_STANDARD_1,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.UserStandardFuncPtr_2=AppCSPS_USER_SEQ_STANDARD_2,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.UserErrorFuncPtr=AppCSPS_USER_SEQ_ERROR,				// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.u32_BlkChkSum=0xFFFFFFFF,								// MUST NOT BE CHANGED!!! -> Ckecksum
                                                #ifdef fFeEXTRAFUNC
                                                #ifdef fFeCANRXTX
                                                .stCntrl.UserRxEventFuncPtr=AppCSPS_USER_SEQ_CAN_RX_Event,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                .stCntrl.UserTxEventFuncPtr=AppCSPS_USER_SEQ_CAN_TX_SynxEvent,	// MUST NOT BE CHANGED!!! -> Func Pointer
                                                #endif
                                                #ifdef fFeRS232RxEvent
                                                .stCntrl.UserRxEventFuncPtr=AppCSPS_USER_SEQ_232_RX_Event,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                #endif
                                                #ifdef fFe100msEvent
                                                .stCntrl.User100msEventFuncPtr=AppCSPS_USER_100ms_Event,		// MUST NOT BE CHANGED!!! -> Func Pointer
                                                #endif
                                                #endif
                                                .stInfo.su.u16_UserPrgVersion=0x1000,							// VERSION of sequence on device
                                                .stInfo.su.u16_UserReserved=0x0000,								// MUST NOT BE CHANGED!!!
                                                .stInfo.su.u8_UserPrgTxt={"Example: Rotate to serial cmd"}		// DESCRIPTION of sequence on device
                                                };


// ---- Globale Variabeln loesen -----------------
LOCATEVARSPS tAPPCSPS stAppCSPS;

//###################################################################
//#		User defines, variables and more for better readability		#
//###################################################################
#define SPEED_1		600
#define SPEED_2		1500
#define SPEED_3		2750

#define u08_state	UVar.u8_StepChain[0]		// State
#define i32_speed	UVar.i32_PosVAR_um_01deg[0]	// Speed

// ---- States --------------------
typedef enum
{
	eSTATE__INIT	= 0,	// Init
	eSTATE__STOP	= 1,	// Rotate 0 (Stop)
	eSTATE__ROTATE	= 2,	// Rotate with speed
	eSTATE__DEFAULT	= 99	// Default
};


/************************************************************************************************/
/*!
 * \brief	SPS-USER function / TaskHandler1 (first)
 * \details	Is called while <SPS-RUN> state every 5ms (not while in error-State!).
 *			Your code shall not block (Cooperative Multitask).
 *			Execution time of your block shall be < 50us / max 0.5ms!
 *
 *			Program here, regularly things, e.g. Checking IO's ....
**************************************************************************************************/
void AppCSPS_USER_SEQ_STANDARD_1(void)
{
	switch (u08_state)
	{
		case eSTATE__INIT:
		{
			// Init timer for sending position
			// 'UVar.u16_Timer5ms[0]' is decremented every 5ms
			UVar.u16_Timer5ms[0] = 20; // 20*5ms=100ms
			break;
		}
		case eSTATE__STOP:
		{
			// Call rotate function
			if (UFu_RotateFuncP(0) == eMS_OK)
			{
				// Send current position
				UFu_SendComUserMsg(UVar.i32_ActualPos_um_01deg);
				u08_state = eSTATE__DEFAULT;
			}
			break;
		}
		case eSTATE__ROTATE:
		{
			// Call rotate function
			if (UFu_RotateFuncP(i32_speed) == eMS_OK)
			{
				u08_state = eSTATE__DEFAULT;
			}
			break;
		}
		default:
		{
			// do something
		}
	}
}

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / TaskHandler2 (second)
 * \details	Is called while <SPS-RUN> state every 5ms (not while in error-State!).
 *			Your code shall not block (Cooperative Multitask).
 *			Execution time of your block shall be < 50us / max 0.5ms!
 *
 *			TaskHandler2, is called 1ms later than TaskHandler1, to allow CPU load splitting into 2 parts.
 *
 *			Program here, regularly things, e.g. Checking IO's ....
**************************************************************************************************/
void AppCSPS_USER_SEQ_STANDARD_2(void)
{
	// Check if timer expired
	if (UVar.u16_Timer5ms[0]==0)
	{
		// Reset timer
		UVar.u16_Timer5ms[0] = 20; // 20*5ms=100ms
		
		// Check if current drive state is "rotate"
		if (UVar.enDrvState == eMS_ROTATE)
		{
			// Send current position over serial interface
			UFu_SendComUserMsg(UVar.i32_ActualPos_um_01deg);
		}
	}
}

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / ErrorHandler
 * \details	Is called once, when core enters <ERROR> state.
 *			Depending on Error, it is not allowed to start here a Moving CMD!
 *
 *			Program here, things to show error, e.g. switch_ON Error-Output
**************************************************************************************************/
void AppCSPS_USER_SEQ_ERROR(void)
{
	// Fill in code...
	mNOP();
}

#ifdef fFeRS232RxEvent
/************************************************************************************************/
/*!
 * \brief	SPS-USER function / RS232-RxEvent on receive of [Df] Command
 * \details	If you react on RS232 Data here, you shall also Clear data receive Structure at the end of the Handler.
 *			To confirm the main firmware Command was handled by your code.
 *
 *			You might use stAppCSPS.SPSUserVar.st32_ComData.u32_data instead of pComData->
 *			pComData->   points to stAppCSPS.SPSUserVar.st32_ComData.u32_data
 
 *			Serial cmd e.g.:	Df#0D#XL4[0x12345678]#CK 				(COMWatch)
 *								0x44 0x66 0x0D 0x78 0x56 0x34 0x12 0x35	(HEX)
 *
 *			Program here your own Command interpreter...
 *
 * \param	pComData - pointer to data received from serial port received data on command [Df]
**************************************************************************************************/
void AppCSPS_USER_SEQ_232_RX_Event(tCSPSCOMRXDATA* pComData)	//!< RW: from serial port received data command <Df>, shall be cleared by USER !)
{
	if(pComData->u32_data == 0)	// Df#0D#XL4[0x00]#CK (0x44 0x66 0x0D 0x00 0x00 0x00 0x00 0x49)
	{
		i32_speed = 0;
		u08_state = eSTATE__STOP;
	}
	else if(pComData->u32_data == 1) // Df#0D#XL4[0x01]#CK (0x44 0x66 0x0D 0x01 0x00 0x00 0x00 0x48)
	{
		i32_speed = SPEED_1;
		u08_state = eSTATE__ROTATE;
	}
	else if(pComData->u32_data == 2) // Df#0D#XL4[0x02]#CK (0x44 0x66 0x0D 0x02 0x00 0x00 0x00 0x47)
	{
		i32_speed = SPEED_2;
		u08_state = eSTATE__ROTATE;
	}
	else if(pComData->u32_data == 3) // Df#0D#XL4[0x03]#CK (0x44 0x66 0x0D 0x03 0x00 0x00 0x00 0x46)
	{
		i32_speed = SPEED_3;
		u08_state = eSTATE__ROTATE;
	}
	
	pComData->u32_data = 0;	// Clear data as handled information
}
#endif

#ifdef fFeCANRXTX
/************************************************************************************************/
/*!
 * \brief	SPS-USER function / CAN-RXEvent on CANopen-PDO4 Rx
 * \details	e.g. MsgId=  0x508  DLC=8  Data = ???
 *
 *			Program here your own command interpreter...
 * 
 * \param	pRxData - Pointer to Data received
 * \param	Datalength - Data count [0..8]
**************************************************************************************************/
void AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
{
   // Fill in code...
   mNOP();
}
#endif

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / CAN-TxSyncEvent on CANopen-PDO4 Tx (Sync on Time OR Sync CMD reception)
 * \details	Collect here your data to send officially at your PDO4-Transaction,
 *			especially if it shall be transmitted synchronized!
 *
 *			Program here your own PDO4-Tx (synchronized Message collector)...
**************************************************************************************************/
#ifdef fFeCANRXTX
void AppCSPS_USER_SEQ_CAN_TX_SynxEvent(void)
{
   // Fill in code...
   mNOP();
}
#endif

/************************************************************************************************/
/*!
 * \brief	SPS-USER function / 100ms Regular Event, called even while in error or homing state
 * \details	Do not define moving actions here... use it e.g. for LED or output signaling
 *
 *			Program here your own LED/output/Transmit Management...
**************************************************************************************************/
#ifdef fFe100msEvent
void AppCSPS_USER_100ms_Event(void)
{
	// Fill in code...
   mNOP();
}
#endif

