/*************************************************************************************************/
/*! \file
\ingroup Appl
\brief    Motor-APP / L2 SPS-Integration / USER Seite

- Integration kundenspezifischer CMDs
- bentigt spezielles Linker Skript
- **** ATTENTION *** needs Firmware Release >= V2.5-000  on all Devices (K11a,K11b,K17a,K17e)

CREATED
\date     3.3.2020
\version  1.0
\author   Michael Zimmermann
\remarks  -

<hr>MODIFIED
\date    -
\version -
\author  -
\remarks -

COPYRIGHT  2020, ADLOS AG, FL-9496 Balzers<hr>
**************************************************************************************************/

#include "L0_TA_datatypes_GCC_ARM.h"                                      // uC Include & Std-Type - Definitions
#include "stm32f0xx.h"
#include "L0_KannMotDrvTypes.h"
#include "L2_APPC_SPS_User.h"

// ---- Internal Definitions ----------------------------

// ---- SPS Control-Block ----------------------
volatile const tAPPCSPS_CNTRLBLK LOCATEUSCNTRL  CNTRLBLKINIT = { {AppCSPS_USER_SEQ_STANDARD_1,                         // Func Pointer
		                                                            AppCSPS_USER_SEQ_STANDARD_2,                         // Func Pointer
		                                                            AppCSPS_USER_SEQ_ERROR,                              // Func Pointer
		                                                            0xFFFFFFFF},                                         // Checksumme
		                                                            0,                                                   // Version
                                                                   {"--DEMO Comm--"} };                                    // USER Text/ Beschreibung

// ---- Globale Variabeln lsen -----------------
tAPPCSPS LOCATEVARSPS stAppCSPS;

// **HEADER_END**             nicht lschen
								   
/************************************************************************************************/
/*!
 *  \brief     SPS-USER Funktion / Demo <4>
 *  \details   Building up an OWN RS232 command interpreter on <Df> instruction
 *             This DEMO needs at least Firmware Version V2.5-000 and KannMOTION-Manager 1.5.x.x
 *
 *  -> we use First Byte of Df Instuction as Command, 2..4 Byte as Extra Data
 *     CMD [0x00]: nothing
 *     CMD [0x01]: GotoValvePosition: DataRange [0..5]     e.g. GotoPos 5         <Df#0D#XL1[0x01]#XL3[5]#CK>
 *     CMD [0x40]: StoreValvePosOffset: Data 24Bit signed  e.g. OffsetPos 21000   <Df#0D#XL1[0x40]#XL3[21000]#CK>
 *     CMD [0x80]: GetActualValvePosition: NoData          e.g. GetValvePos       <Df#0D#XL1[0x80]#XL3[0]#CK>
 *     CMD [0x81]: ChangeGotoPos Max-Speed / Data 1..200  in HalfPercent of MaxSpeed Param e.g. Comwatch Terminal: <Df#0D#XL1[0x81]#XL1[100]#XL2[0]#CK>
 *
 *  USAGE:
 *    stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0]       : as Helping Position Calculation Value
 *    stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0]   : Valve 0-Position Offset Absolute in 0.1, NonVolatile
 *
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      3.3.2020
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
  // -- First Time, Check NV-Vars if Content is ok, otherwise set to 0
  if (stAppCSPS.SPSUserVar.u8_StepChain[0]==0)
  {
     if ((stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0] < -7200) || (stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0] > 7200))
     {
       stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0] = 0;
     }
     stAppCSPS.SPSUserVar.u8_StepChain[0]++;
  }

  // -- RS232 -- Befehlsinterpreter
  switch (stAppCSPS.SPSUserVar.st32_ComData.u8_CMD)
  {
     // --- nothing to do
     case 0:
     {
        break;
     }
     // --- CMD [1]: GotoValvePosition: Data: [0..5] e.g. Comatch Terminal: GotoPos 3  <Df#0D#XL1[0x01]#XL3[3]#CK>
     //              Walve Positions are at every 60, so gives a multiplication Factor of 600 in 0.1 Mode
     case 0x01:
     {
        // Range Limit check
        if (stAppCSPS.SPSUserVar.st32_ComData.u8_Data > 5)
        {
           stAppCSPS.SPSUserVar.st32_ComData.u8_Data = 5;
        }
        // Calculate Valve Position to Goto, CMD-Data * 600 + Offset
        stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] = stAppCSPS.SPSUserVar.st32_ComData.u8_Data;
        stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] *= 600;
        stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] += stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0];
        // Execute, and Reset Full Command Info, if CMD accepted
        if (stAppCSPS.SPSCallFunctions.GotoFuncP(stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0],eGOTO_um_01deg)==eMS_OK) // Call Goto Function
        {
           stAppCSPS.SPSUserVar.st32_ComData.u32_data = 0;                                                                 // Clear full Message
        }
        break;
     }
     // --- CMD [0x40]: StoreValvePosOffset: Data 24Bit signed  e.g. OssetPos 3600    <Df#0D#XL1[0x40]#XL3[3600]#CK>
     //                 !! ATTENTION !!: do not execute stAppCSPS.SPSCallFunctions.StoreUserData() very often
     //                                  resp. regularly because of data retention restrictions of NV data
     case 0x40:
     {
        // Take over 24-Bit data, Adapt signed value from 24 to 32Bit
        stAppCSPS.SPSUserVar.KxxFlexUser.u32_UserVar[0] = stAppCSPS.SPSUserVar.st32_ComData.u32_data;                     // Get Full Command
        stAppCSPS.SPSUserVar.KxxFlexUser.u32_UserVar[0] >>= 8;                                                            // reduce to Data Content 32->24Bit
        if (stAppCSPS.SPSUserVar.KxxFlexUser.u32_UserVar[0] & 0x00800000)                                                 // Check Signed Bit, if set value is negative
        {
          stAppCSPS.SPSUserVar.KxxFlexUser.u32_UserVar[0] |= 0xFF000000;                                                  // make correct Adaption 24->32Bit signed
        }
        // Store it in NV-Structure, and Reset Full Command Info, if CMD accepted
        stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0] = (SI_32) stAppCSPS.SPSUserVar.KxxFlexUser.u32_UserVar[0];
        if (stAppCSPS.SPSCallFunctions.StoreUserData()==eMS_OK)
        {
           stAppCSPS.SPSUserVar.st32_ComData.u32_data = 0;                                                                // Clear full Message
        }
        break;
     }
     // --- CMD = [0x80] : GetActualValvePosition: NoData    e.g. Comwatch Terminal: <Df#0D#XL1[0x80]#XL3[0]#CK>
     //                    Returns calculated Valve Position back
     case 0x80:
     {
        // Calculate Valve Position
        stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] = stAppCSPS.SPSUserVar.i32_ActualPos_um_01deg;                    // Set actual Position
        stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] -= stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0];           // Take out Offset
        stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] += 300;                                                           // Add Half Valve Distance ( bacause of Rounding )
        stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] /= 600;                                                           // direct Divison, not recommended
        // Send Answer MSG, and Reset Full Command Info, if CMD accepted
        if (stAppCSPS.SPSCallFunctions.SendComUserMsg(stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0])==true)             // Send a <DF>cr<Data32><CK> back
        {
           stAppCSPS.SPSUserVar.st32_ComData.u32_data = 0;                                                                // Clear Message
        }
        break;
     }
     // --- CMD = [0x81] : ChangeGotoPos Max-Speed / Data 1..200  in HalfPercent of MaxSpeed Param e.g. Comwatch Terminal: <Df#0D#XL1[0x81]#XL1[100]#XL2[0]#CK>
     //                    Returns calculated Valve Position back
     case 0x81:
     {
        stAppCSPS.SPSCallFunctions.DrvSetMaxSpeed(stAppCSPS.SPSUserVar.st32_ComData.u8_Data);
        stAppCSPS.SPSUserVar.st32_ComData.u32_data = 0;                                                                    // Clear Message
        break;
     }
     // --- Default: unknown command, Clear Message to allow reception of new Commands, send Error Frame
     default:
     {
        stAppCSPS.SPSUserVar.st32_ComData.u32_data = 0;                                                                   // Clear Message
        stAppCSPS.SPSCallFunctions.SendComUserMsg(0x80000000);                                                            // Send a <DF>cr<Data32><CK> back
        break;
     }
  }
}

void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
}

/************************************************************************************************/
/*!
 *  \brief     Eigentlichen SPS-USER Funktion ( Fehlerablauf ) // Leer Vorlage
 *  \details
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
  mNOP();
  mNOP();
  mNOP();
  mNOP();
}